/* Maintainance of binary tree containing clients */
/* Considering linked lists */

#include "client.h"

extern long int clients;

/* Pointer to root node */
static struct clientinfo * root=NULL;

/* Add a client to the tree */
int add_client(char *nick,
	       char *user,
	       char *host)
{
	struct clientinfo *current;
	struct clientinfo *client;
	int str_cmp;

	client=malloc(sizeof(struct clientinfo));
	if(client==NULL) {
		return CLIENT_FAILED;
	} else {
		strncpy(client->nick,nick,NICK_LENGTH);
		strncpy(client->user,user,USER_LENGTH);
		strncpy(client->host,host,HOST_LENGTH);
		client->chan_start=NULL;
		return add_client_struct(client);
	}
}
	

int add_client_struct(struct clientinfo *client)	
{	
	struct clientinfo *current;
	int str_cmp,i;
		
	/* Set some variables */
	client->left=NULL;
	client->right=NULL;
	
	current=root;
	
	/* Actually add client to tree */
	if(root==NULL) {
		/* If there are no records, add at root */
		root=client;
		client->parent=NULL;
		return CLIENT_OK;
	} else {
		/* Else get to where we need to be */
		while(1) {
			str_cmp=lc_strncmp(client->nick,current->nick,NICK_LENGTH);	
			if(str_cmp>0) {
				if(current->right==NULL) {
					current->right=client;
					client->parent=current;
					return CLIENT_OK;
				} else {
					current=current->right;
				}
			} 
			if(str_cmp<0) {
				if(current->left==NULL) {
					current->left=client;
					client->parent=current;
					return CLIENT_OK;
				} else {
					current=current->left;
				}
			}
			if(str_cmp==0) {
				return CLIENT_FAILED;
			}
		}
	}
}

/* Delete a leaf node */
static int rm_leaf_node(struct clientinfo *parent,
		        struct clientinfo *remove)
{
	if(root!=remove) {
		if(parent->right==remove) {
			parent->right=NULL;
		} else {
			parent->left=NULL;
		}
	} else {
		root=NULL;
	}
	free(remove);

	return CLIENT_OK;
}

/* delete a node with only one child */
static int rm_single_child(struct clientinfo *parent,
			   struct clientinfo *remove, 
			   struct clientinfo *child)
{
	if(root!=remove) {
		if(parent->left==remove) {
			parent->left=child;
		} else {
			parent->right=child;
		}
		
		child->parent=parent;
	} else {
		root=child;
		child->parent=NULL;
	}
		
	free(remove);
	return CLIENT_OK;	
}

/* delete a node with two children */
/* *shudder*	*/

static int rm_two_kids(struct clientinfo *parent,
		       struct clientinfo *remove)
{
	struct clientinfo *replacement;
	struct clientinfo *current;
	int end=0;
	
	/* find replacement node */
	current=remove->left;
	while(end==0) {
		if(current->right!=NULL) {
			current=current->right;
		} else {
			replacement=current;
			end=1;
		}
	}
	
	/* Set pointer in parent to point to replacement */
	if(remove->parent!=NULL && parent->right==remove) {
		parent->right=replacement;
	} 
	if(remove->parent!=NULL && parent->left==remove) {
		parent->left==replacement;
	}
	if(remove->parent==NULL) {
		root=replacement;
	}

	/* set replacement parent pointer to null */
	if(replacement->parent->right==replacement) {
		replacement->parent->right=NULL;
		if(replacement->left!=NULL) 
			replacement->parent->right=replacement->left;
	}
	if(replacement->parent->left=replacement) {
		replacement->parent->left=NULL;
		if(replacement->left!=NULL) 
			replacement->parent->left=replacement->left;
	}

	/* set right pointer of replacement pointer to same as outgoing one */
	replacement->right=remove->right;
	replacement->parent=remove->parent;
	replacement->right->parent=replacement;
	if(replacement->left!=NULL) 
		replacement->left->parent=replacement;
	
	
	/* Free deleted node */
	free(remove);

	return CLIENT_OK;
}

/* Function to delete a node */
/* Calls other functions to actually delete */
int remove_client(char *nick){
	struct clientinfo * current;
	int str_res;
	
	/* check tree actually contains something */
	if(root==NULL || nick[0]=='\0') 
		return CLIENT_FAILED;
	
	current=root;
	while(1) {
		str_res=lc_strncmp(nick,current->nick,NICK_LENGTH);
		
		if(str_res==0) {
			/* Found the correct item */
			if(current->left==NULL && current->right==NULL) 
				/* leaf node */
				return rm_leaf_node(current->parent,current);
				
			if(current->left!=NULL && current->right==NULL) 
				/* Node with a left child */
				return rm_single_child(current->parent,current,current->left);

			if(current->right!=NULL && current->left==NULL)
			       /* Node with a right child */	
				return rm_single_child(current->parent,current,current->right);

			if(current->right!=NULL && current->left!=NULL)
				/* node with two children */
				return rm_two_kids(current->parent,current);
		}
	       	/* Move pointers along */	
		if(str_res<0)
			current=current->left;
		
		if(str_res>0)
			current=current->right;	
		
		/* Drop out if we reach the end */
		if(current==NULL)
			return CLIENT_FAILED;

	}
}

/* Function to find a specified user */
struct clientinfo * locate_client(char *user)
{
	struct clientinfo *current;
	int str_cmp;
	
	current=root;

	if(current==NULL) 
		return NULL;
	
	while(1) {
		str_cmp=lc_strncmp(user,current->nick,NICK_LENGTH);
		
		if(str_cmp==0) 
			return current;
		
		if(str_cmp>0) {
			if(current->right==NULL)
				return NULL;
			current=current->right;
		}
		if(str_cmp<0) {
			if(current->left==NULL)
				return NULL;
			current=current->left;
		}
	}		
}	

/* recursive printing algorithm for debugging */
static void print_node(struct clientinfo *node)
{
	if(node!=NULL) {
		if(node->left==NULL && node->right==NULL) {
			printf("%s - parent %#x - left %#x - right %#x - %#x\n",node->nick,
										node->parent,node->left,
										node->right,node);
		} else {
			if(node->left!=NULL)
				print_node(node->left);
			if(node->right!=NULL)
				print_node(node->right);
			printf("%s - parent %#x - left %#x - right %#x - %#x\n",node->nick,
										node->parent,node->left,
										node->right,node);
		}
	}
}	
		
/* debugging only */
int list_tree()
{
	print_node(root);
}
