#include "cmd_n.h"

/* Search for a user in the holding linked list */
static int user_in_list(struct clientinfo *temp_start,char * nick)
{
	struct clientinfo *current;
	
	current=temp_start;
	
	while(current!=NULL) {
		if(lc_strncmp(current->nick,nick,-1)==0)
			return TRUE;

		if(current->list_next!=NULL) 
			current=current->list_next;
	}
	return FALSE;
}

/* Inform client if nick is in use */
static int nick_taken(struct clientinfo *user_info,char *requested_nick) 
{
	char buffer[80];
	
	ircd_sprintf(buffer,80,"%s :Nickname already in use",requested_nick);
	if(user_info->nick[0]!='\0') {
		return send_numeric(user_info->fd,user_info->nick,433,buffer);
	} else {
		return send_numeric(user_info->fd,"*",433,buffer);
	}
}

/* inform room of nick change */
static int inform_users(struct clientinfo *user_info,char *requested_nick)
{
	char buffer[100];
	struct user_wrap *curr_user;
	struct chan_wrap *curr_chan;
	
	/* Inform client itself */
	ircd_sprintf(buffer,100,"%s %s","NICK",requested_nick);
	send_to_fd(user_info->fd,user_info,buffer);

	/* Inform rooms */
	if(user_info->chan_start==NULL) {
		return EXIT_SUCCESS;
	} else {
		curr_chan=user_info->chan_start;
		while(curr_chan!=NULL) {
			curr_user=curr_chan->chan_info->users_start;
			while(curr_user!=NULL) {
				/* Miss out user changing nick */
				if(curr_user->user_info!=user_info)
					send_to_fd(curr_user->user_info->fd,user_info,buffer);
				curr_user=curr_user->next;
			}
			curr_chan=curr_chan->next;
		}
		return EXIT_SUCCESS;
	}
	
		
}
static int update_tree(struct clientinfo **user_info,
		       char *new_nick,
		       struct clientinfo *fd_list[])
{
	struct clientinfo *new;
	struct chan_wrap *curr_chan;
	struct user_wrap *curr_client;
	
	/* Allocate memory and copy from old to new */
	new=malloc(sizeof(struct clientinfo));
	memcpy(new,(*user_info),sizeof(struct clientinfo));
	
	/* Update channel pointers */
	curr_chan=new->chan_start;
	while(curr_chan!=NULL) {
		curr_client=curr_chan->chan_info->users_start;
		while(curr_client!=NULL) {
			if(curr_client->user_info==(*user_info)) {
				curr_client->user_info=new;
				break;
			} else {
				curr_client=curr_client->next;
			}
		}
		curr_chan=curr_chan->next;
	}
					
	
	/* Remove client from tree */
	remove_client((*user_info)->nick);
	
	/* Update various arrays and structs */
	(*user_info)=new;
	fd_list[(*user_info)->fd]=(*user_info);
	
	
	/* Copy new nick */
	strncpy((*user_info)->nick,new_nick,NICK_LENGTH);
	
	/* Add back into tree */
	return add_client_struct(*user_info);
}
	
/* handles NICK commands */
int cmd_nick(struct clientinfo **user_info,
	     struct parsed_cmd *cmd,
	     struct clientinfo *temp_start,
	     struct clientinfo *fd_list[])
{
	struct clientinfo *current;
	struct clientinfo *exist;
	char new_nick[NICK_LENGTH];
	char buffer[100];
	int i;
	
	/* Check we actually have a nick */
	if(cmd->params[0]!=NULL || cmd->text!=NULL) {
		/* Let it use either text or params[0] */
		if(cmd->params[0]==NULL) {
			strncpy(new_nick,cmd->text,NICK_LENGTH);
		} else {
			strncpy(new_nick,cmd->params[0],NICK_LENGTH);
		}
		/* Check validity of new nick */
		if(!check_nick(new_nick)) {
			ircd_sprintf(buffer,100,"%s :Erroneous Nick Name",new_nick);
			send_numeric((*user_info)->fd,(*user_info)->nick,432,buffer);
			cmd_free(cmd);
			return EXIT_FAILURE;	
		}
			
		exist=locate_client(new_nick);
		if(exist==NULL) {
			/* User isn't in tree */
			if(user_in_list(temp_start,new_nick)) {
				/* User isn't in list */
				if(((*user_info)->flags & USR_LIST)==0) {
					/* User is logged in, have to update and inform */
					inform_users((*user_info),new_nick);
					update_tree(user_info,new_nick,fd_list);
				} else {
					/* User is in list, just set nick */
					strncpy((*user_info)->nick,new_nick,NICK_LENGTH);
				}
				cmd_free(cmd);
				return EXIT_SUCCESS;
			} else {
				/* User is in list */
				/* Inform User */
				nick_taken((*user_info),new_nick);
				cmd_free(cmd);
				return EXIT_SUCCESS;
			}
		} else {
			nick_taken((*user_info),new_nick);
			cmd_free(cmd);
			return EXIT_SUCCESS;
		}
	} else {
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
	
	
}



/* called to change name */
int cmd_name(struct clientinfo *user_info,struct parsed_cmd *cmd)
{
	if(cmd->params[0]!=NULL) 
		strncpy(user_info->user,cmd->params[0],USER_LENGTH);
	
	cmd_free(cmd);
}

/* check validity of nicks, uses ascii codes */
static int check_nick(char *nick)
{
	int i;
	
	/* Check first chars */
	switch (nick[0]) {
		case 48: 
		case 49: 
		case 50: 
		case 51:
		case 52:
		case 53:
		case 54:
		case 55:
		case 56:
		case 57:
		case 45:
			/* These aren't allowed */
			return 0;
			break;
		default:
			break;
	}	
			
	
	/* Check individual characters */
	for(i=0;nick[i]!='\0';i++) {
		/* Huge if structure */
		if( (nick[i] > 47 && nick[i] < 58) || 
		    (nick[i] > 64 && nick[i] < 125) ||
		    (nick[i] == 45) ) {

		} else {
			/* Invalid char */
			return 0;
		}
	}

	/* Return true if nick is ok */	
	return 1;
}
