#include "cmd_privmsg.h"

/* cmd_privmsg - deals with privmsg system */
int cmd_privmsg(struct clientinfo *sender,struct parsed_cmd *cmd)
{
	struct clientinfo *recip;
	struct channel *chan;
	struct user_wrap *curr_client;
	char buffer[BUF_SIZE];
	
	/* find out if it's a client or a room */
	if(cmd->params[0]==NULL) {
		ircd_sprintf(buffer,BUF_SIZE,"%s : Not enough parameters\n",cmd->command);
		send_numeric(sender->fd,sender->nick,461,buffer);
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
		
	/* Attempt to locate client */
	recip = locate_client(cmd->params[0]);

	if(recip==NULL) {
		if((chan=locate_channel(cmd->params[0]))==NULL) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :No such nick/channel",cmd->params[0]);
			send_numeric(sender->fd,sender->nick,401,buffer);
			cmd_free(cmd);
			return EXIT_SUCCESS;
		} else {
			/* send to a room */
			privmsg_to_channel(sender,chan,cmd->text);
			cmd_free(cmd);
			return EXIT_SUCCESS;
		}
	} else {
		/* Send to a client */
		if(privmsg_to_user(sender,recip,cmd->text)==EXIT_SUCCESS) {
			cmd_free(cmd);
			return EXIT_SUCCESS;
		} else {
			cmd_free(cmd);
			return EXIT_FAILURE;
		}
		
	}
}

/* send a privmsg to a user */
static int privmsg_to_user(struct clientinfo *sender,
			   struct clientinfo *recip,
			   char *text)
{
	char cmd_buff[500];
	
	/* form end of string */
	if(text==NULL) {
		return EXIT_FAILURE;
	} else {
		ircd_sprintf(cmd_buff,BUF_SIZE,"PRIVMSG %s :%s",recip->nick,text);
	
		/* call procedure to send to client */
	 	return	send_to_fd(recip->fd,sender,cmd_buff);
	}
}

/* send a privmsg to a channel */
static int privmsg_to_channel(struct clientinfo *sender,
			      struct channel *chan,
			      char *text)
{
	struct user_wrap *curr_user;
	int found=0;
	/* Max size of irc command */
	char buffer[512];
	
	/* Find out if user is on that channel */
	curr_user=chan->users_start;
	while(curr_user!=NULL && found==0) {
		if(curr_user->user_info==sender) {
			found=1;
		} else {
		       curr_user=curr_user->next;
		}
	}
	if(found==1 || !(chan->mode & CMODE_MEMONLYSEND)) {
		/* Send message */
		ircd_sprintf(buffer,BUF_SIZE,"PRIVMSG %s :%s",chan->name,text);
		curr_user=chan->users_start;
		while(curr_user!=NULL) {
			if(curr_user->user_info!=sender) {
				send_to_fd(curr_user->user_info->fd,sender,buffer);
			}
			curr_user=curr_user->next;
		}
		return EXIT_SUCCESS;
	} else {
		/* Tell them they don't have permission */
		ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot send to channel",chan->name);
		send_numeric(sender->fd,sender->nick,404,buffer);
		return EXIT_SUCCESS;		
	}
}
