#include "cmd_topic.h"

int cmd_topic(struct clientinfo *sender,struct parsed_cmd *cmd)
{
	struct channel *chan;
	char buffer[BUF_SIZE];
	struct user_wrap *cli_wrap;
	
	if(cmd->params[0]==NULL) {
		/* Not enough params */
		cmd_free(cmd);
		return EXIT_FAILURE;
	}	
	
	chan=locate_channel(cmd->params[0]);
	
	if(chan==NULL) {
		/* Invalid Channel */
		ircd_sprintf(buffer,BUF_SIZE,"%s :No such channel",cmd->params[0]);
		send_numeric(sender->fd,sender->nick,403,buffer);
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
	
	/* Check we are on channel */
	cli_wrap=check_on(sender,chan);
	if(cli_wrap==NULL) {
		/* User isn't on that channel */
		ircd_sprintf(buffer,BUF_SIZE,"%s :You're not on that channel",chan->name);
		send_numeric(sender->fd,sender->nick,442,buffer);
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
		
	if(cmd->had_text==0 && cmd->text==NULL) {
		/* Report topic */
		if(chan->topic[0]=='\0') {
			ircd_sprintf(buffer,BUF_SIZE,"%s :No topic is set",chan->name);
			send_numeric(sender->fd,sender->nick,331,buffer);
		} else {
			ircd_sprintf(buffer,BUF_SIZE,"%s :%s",chan->name,chan->topic);
			send_numeric(sender->fd,sender->nick,332,buffer);
			ircd_sprintf(buffer,BUF_SIZE,"%s %s %ld",chan->name,chan->topic_set,
						  chan->topic_settime);
			send_numeric(sender->fd,sender->nick,333,buffer);
		}
		cmd_free(cmd);
		return EXIT_SUCCESS;
	}
	
	/* Check for ops */
	if( (chan->mode & CMODE_TOPIC) && 
	     		!(cli_wrap->chan_mode & CHANUSR_OP) ){
	     	/* Topic is protected and they haven't got ops */
		ircd_sprintf(buffer,BUF_SIZE,"%s :Ha! You're not a chan op",chan->name);
		send_numeric(sender->fd,sender->nick,482,buffer);
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
	
	
	/* Change topic */
	if(cmd->text!=NULL) {
		/* Set Topic */
		if(cmd->text[0]=='\0') {
			/* Clear Topic */
			chan->topic[0]='\0';
		} else {
			/* Set topic to cmd->text */
			strncpy(chan->topic,cmd->text,TOPIC_MAX_LENGTH);
		}
		chan->topic_settime=time(NULL);
		strncpy(chan->topic_set,sender->nick,NICK_LENGTH);
		inform_room_top_change(sender,chan);
		cmd_free(cmd);
		return EXIT_SUCCESS;
	} else {
		cmd_free(cmd);
		return EXIT_FAILURE;
	}
}

static int inform_room_top_change(struct clientinfo *changer,
				  struct channel *chan)
{
	struct user_wrap *curr_client;
	char buffer[TOPIC_MAX_LENGTH + 100];
	
	if(chan->topic[0]=='\0') {
		/* Topic cleared */
		ircd_sprintf(buffer,TOPIC_MAX_LENGTH+100,"TOPIC %s :",chan->name);
	} else {
		ircd_sprintf(buffer,TOPIC_MAX_LENGTH+100,"TOPIC %s :%s",chan->name,chan->topic);
	}

	curr_client=chan->users_start;
	while(curr_client!=NULL) {
		send_to_fd(curr_client->user_info->fd,changer,buffer);
		curr_client=curr_client->next;
	}
	
}

static struct user_wrap *check_on(struct clientinfo *client,
				  struct channel *chan)
{
	struct user_wrap *curr_client;
	
	curr_client=chan->users_start;

	while(curr_client!=NULL) {
		if(curr_client->user_info==client) {
			return curr_client;
		} else {
			curr_client=curr_client->next;
		}
	}

	return NULL;
}
